
using System;
using System.Reflection;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using DotSpatial.Controls;
using DotSpatial.Data;
using DotSpatial.Serialization;
using DotSpatial.Symbology;
using GeoAPI.Geometries;
using Point = System.Drawing.Point;
using GPoint = NetTopologySuite.Geometries.Point;
using DotSpatial.NTSExtension;
using NetTopologySuite.Geometries;
using NetTopologySuite.Geometries.Utilities;
using log4net;


namespace DotSpatial.Plugins.OverviewMap
{
    public class MoveMapAreaFunction : MapFunction
    {
        #region Fields

        private static readonly log4net.ILog logger = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the MoveVertexFunction class where the Map will be defined.
        /// </summary>
        /// <param name="map">The map control that implements the IMap interface.</param>
        public MoveMapAreaFunction(IMap map) : base(map)
        {
            Configure();
            this.BusySet = false;
        }

        #endregion


        #region Properties

        /// <summary>
        /// Gets or sets the layer.
        /// </summary>
        public IFeatureLayer ActiveLayer { get; set; }
       // public IFeature ActiveFeature { get; set; }
        public bool IsDragging { get; set; }
        public Coordinate DragCoord { get; set; }
        public IGeometry DragGeometry { get; set; }
        public IMap MainMap { get; set; }
        public Rectangle DragView { get; set; }
        public Rectangle MoveView { get; set; }
        public Extent MoveExtent { get; set; }

        private Timer ZoomTimer { get; set; }
        public bool BusySet { get; set; }


        #endregion

        #region Methods


        /// <summary>
        /// Deselects the selected feature and removes the highlight from any highlighted feature.
        /// </summary>
        public void ClearSelection()
        {
        }

        private void Configure()
        {
            YieldStyle = (YieldStyles.LeftButton);
            this.ZoomTimer = new Timer { Interval = 100 };
            this.ZoomTimer.Tick += ZoomTimerTick;
        }

        private void ZoomTimerTick(object sender, EventArgs e)
        {
            this.ZoomTimer.Stop();
            if (this.MainMap.MapFrame == null) { return;}
          //  this.MainMap.MapFrame.ResetExtents();
            this.Map.IsBusy = false;
            this.MainMap.IsBusy = false;
            this.BusySet = false;
        }

        protected override void OnActivate()
        {
            base.OnActivate();
        }

        /// <inheritdoc />
        protected override void OnDeactivate()
        {
            base.OnDeactivate();
        }

        /// <inheritdoc />
        protected override void OnMouseDown(GeoMouseArgs e)
        {
            if (this.ActiveLayer.DataSet.Features.Count == 0)
            {
                return;
            }

            if (e.Button == MouseButtons.Left)
            {
                if (this.IsMouseOver(e))
                {
                    var feature = this.ActiveLayer.DataSet.Features[0];
                    this.DragCoord = e.GeographicLocation;
                    this.DragGeometry = feature.Geometry;
                    this.IsDragging = true;
                    this.Map.IsBusy = true;
                    this.MainMap.IsBusy = true;
                    this.DragView = this.MainMap.MapFrame.View;
                }
                else
                {
                    this.IsDragging = false;
                }
            }
            base.OnMouseDown(e);
        }

        /// <inheritdoc />
        protected override void OnMouseMove(GeoMouseArgs e)
        {
            this.ZoomTimer.Stop();
            if (this.IsDragging)
            {
                var maxPolygon = this.MainMap.GetMaxExtent().ToEnvelope().ToPolygon();

                // Begin snapping changes
                Coordinate c0 = e.GeographicLocation;

                var moveX = c0.X - this.DragCoord.X;
                var moveY = c0.Y - this.DragCoord.Y;


                var trans = AffineTransformation.TranslationInstance(moveX, moveY);
                var moveGeom = trans.Transform(this.DragGeometry);

                if (maxPolygon.Contains(moveGeom))
                {
                    this.ActiveLayer.DataSet.Features.Clear();
                    this.ActiveLayer.DataSet.AddFeature(moveGeom);

                    this.Map.MapFrame.Initialize();
                    this.MainMap.ViewExtents.X = moveGeom.Coordinates[0].X;
                    this.MainMap.ViewExtents.Y = moveGeom.Coordinates[0].Y;
                }
            }
            else
            {
                this.Map.Cursor = this.IsMouseOver(e) ? Cursors.SizeAll : Cursors.Default;
            }

            base.OnMouseMove(e);
        }

        /// <inheritdoc />
        protected override void OnMouseUp(GeoMouseArgs e)
        {
            if (e.Button == MouseButtons.Left && this.IsDragging)
            {
                this.IsDragging = false;

                this.MainMap.MapFrame.ResetExtents();
                this.Map.IsBusy = false;
                this.MainMap.IsBusy = false;
                this.BusySet = false;
            }
          //  Map.MapFrame.Initialize();
            base.OnMouseUp(e);
        }
        protected bool IsMouseOver(GeoMouseArgs e)
        {
            bool ret = false;
            if(this.ActiveLayer.DataSet.Features.Count > 0)
            {
                var gpoint = new GPoint(e.GeographicLocation);
                var feature = this.ActiveLayer.DataSet.Features[0];
                var polygon = new Polygon((ILinearRing)feature.Geometry);
                ret = polygon.Intersects(gpoint);
            }

            return ret;
        }

        #endregion
    }
}