﻿using DevExpress.Utils;
using DevExpress.XtraBars.Ribbon;
using DevExpress.XtraEditors;
using DevExpress.XtraTreeList.Nodes;
using DotSpatial.Controls;
using DotSpatial.Controls.Docking;
using DotSpatial.Data;
using DotSpatial.NTSExtension;
using DotSpatial.Plugins.OverviewMap.Properties;
using DotSpatial.Serialization;
using DotSpatial.Symbology;
using GeoAPI.Geometries;
using log4net;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace DotSpatial.Plugins.OverviewMap
{
    public class OverviewMapPlugin : Extension
    {
        /// <summary>ログ(log4net)</summary>
        private static readonly log4net.ILog logger =
            LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

        private GroupControl ovmapPanel;

        public OverviewMapPlugin()
        {
            DeactivationAllowed = false;
        }


        private IMapFeatureLayer MapAreaLayer { get; set; }
        private MoveMapAreaFunction MoveMapAreaFunction { get; set; }
        private IMap OverviewMap 
        { 
            get
            {
                return (IMap)((Map)this.App.Map).Tag;
            }
            set
            {
                ((Map)this.App.Map).Tag = value;
            }
        }
        

        #region Public Methods

        /// <summary>
        /// Initialize the DotSpatial plugin
        /// </summary>
        public override void Activate()
        {

            this.CreateOverviewMap();

             this.App.Map.MapFrame.ViewExtentsChanged += OnMapViewExtentsChanged;
            //        this.App.Map.MapFrame.Layers.ItemChanged += OnItemChanged;

            this.OverviewMap.MapFunctions.Clear();
            this.OverviewMap.FunctionMode = FunctionMode.None;
            this.OverviewMap.Projection = this.App.Map.Projection;
            this.OverviewMap.MapFrame.ProjectionModeDefine = ActionMode.Always;

            this.MoveMapAreaFunction = new MoveMapAreaFunction(this.OverviewMap);
            this.MoveMapAreaFunction.MainMap = this.App.Map;
            this.OverviewMap.MapFunctions.Add(this.MoveMapAreaFunction);

            this.LoadOverViewMap();
            base.Activate();
        }

        private void CreateOverviewMap()
        {
            var map = (Map)this.App.Map;
            map.Resize += map_Resize;
            var form = (Form)map.Parent;
            form.LocationChanged += form_LocationChanged;
            form.Shown += form_Shown;

            var ovmap = new Map();
            ovmap.AllowDrop = true;
            ovmap.BackColor = System.Drawing.Color.White;
            ovmap.CollectAfterDraw = false;
            ovmap.CollisionDetection = false;
            ovmap.ExtendBuffer = false;
            ovmap.FunctionMode = DotSpatial.Controls.FunctionMode.None;
            ovmap.IsBusy = false;
            ovmap.IsZoomedToMaxExtent = false;
            ovmap.Legend = null;
            ovmap.Name = "map1";
            ovmap.ProgressHandler = null;
            ovmap.ProjectionModeDefine = DotSpatial.Controls.ActionMode.Prompt;
            ovmap.ProjectionModeReproject = DotSpatial.Controls.ActionMode.Prompt;
            ovmap.RedrawLayersWhileResizing = false;
            ovmap.SelectionEnabled = true;
            ovmap.TabIndex = 4;
            ovmap.Tag = "OverviewMap";
            ovmap.ZoomOutFartherThanMaxExtent = false;
            ovmap.Dock = DockStyle.Fill;
            this.OverviewMap = ovmap;

            this.ovmapPanel = new GroupControl();
            this.ovmapPanel.Dock = DockStyle.None;
            this.ovmapPanel.Controls.Add(ovmap);
            this.ovmapPanel.Size = new System.Drawing.Size(150, 150);
            this.ovmapPanel.Location = new System.Drawing.Point(map.Size.Width - this.ovmapPanel.Size.Width, 0);
            this.ovmapPanel.Padding = new System.Windows.Forms.Padding(3);
            this.ovmapPanel.ShowCaption = false;

            var rc = new ResizeableControl(this.ovmapPanel);
            map.Controls.Add(this.ovmapPanel);
        }

        private void map_Resize(object sender, System.EventArgs e)
        {
            this.ResetLocation();
        }
        private void form_LocationChanged(object sender, System.EventArgs e)
        {
            this.ResetLocation();
        }
        private void form_Resize(object sender, System.EventArgs e)
        {
            this.ResetLocation();
        }
        private void form_Shown(object sender, System.EventArgs e)
        {
            this.ResetLocation();
           // this.ovmapForm.Show();
        }

        private void ResetLocation()
        {
            var map = (Map)this.App.Map;
            //var scrnLocation = map.PointToScreen(map.Location);
            //this.ovmapForm.Location = new System.Drawing.Point(scrnLocation.X + map.Size.Width - this.ovmapForm.Size.Width, scrnLocation.Y);

            this.ovmapPanel.Location = new System.Drawing.Point(map.Size.Width - this.ovmapPanel.Size.Width, 0);

        }


        /// <summary>
        /// Fires when the plugin should become inactive
        /// </summary>
        public override void Deactivate()
        {
            this.App.Map.MapFrame.ViewExtentsChanged -= OnMapViewExtentsChanged;

            this.App.HeaderControl.RemoveAll();
            this.App.DockManager.Remove("kOvMap");

            base.Deactivate();
        }

        private void LoadOverViewMap()
        {
            var file = ConfigurationManager.AppSettings["overview_map.dspx"];
            var mLayers = this.DeserializeLayers(file);
            this.OverviewMap.Layers.Clear();
            this.OverviewMap.Layers.AddRange(mLayers);

            this.MapAreaLayer = this.OverviewMap.Layers.Add(new FeatureSet(FeatureType.Line));
            ((LineSymbolizer)this.MapAreaLayer.Symbolizer).SetFillColor(Color.Red);
            ((LineSymbolizer)this.MapAreaLayer.Symbolizer).SetWidth(2);

            //var linearRing = featureSet.Extent.ToEnvelope().ToLinearRing();
            var linearRing = this.OverviewMap.Extent.ToEnvelope().ToLinearRing();
            this.MapAreaLayer.DataSet.AddFeature(linearRing);

            this.MoveMapAreaFunction.ActiveLayer = this.MapAreaLayer;
            this.MoveMapAreaFunction.Activate();

            this.OverviewMap.ZoomToMaxExtent();

            this.OverviewMap.Resized += OnResized;

        }

        private void OnResized(object sender, EventArgs args)
        {
            //this.MapAreaLayer.DataSet.Features.Clear();
            //var linearRing = this.App.Map.Extent.ToEnvelope().ToLinearRing();
            //this.MapAreaLayer.DataSet.AddFeature(linearRing);

            const double eps = 1e-7;
            Envelope layerEnvelope = this.OverviewMap.ViewExtents.ToEnvelope();
            if (layerEnvelope.Width > eps && layerEnvelope.Height > eps)
            {
                layerEnvelope.ExpandBy(layerEnvelope.Width / 10, layerEnvelope.Height / 10); // work item #84
            }
            else
            {
                double zoomInFactor = 0.05; //fixed zoom-in by 10% - 5% on each side
                double newExtentWidth = this.OverviewMap.ViewExtents.Width * zoomInFactor;
                double newExtentHeight = this.OverviewMap.ViewExtents.Height * zoomInFactor;
                layerEnvelope.ExpandBy(newExtentWidth, newExtentHeight);
            }

            this.OverviewMap.ViewExtents = layerEnvelope.ToExtent();
        }


        private void OnMapViewExtentsChanged(object sender, ExtentArgs args)
        {

            if (this.OverviewMap.Layers.Count == 0 || this.MapAreaLayer == null)
            {
                return;
            }

            this.MapAreaLayer.DataSet.Features.Clear();
            var linearRing = args.Extent.ToEnvelope().ToLinearRing();
            this.MapAreaLayer.DataSet.AddFeature(linearRing);

            if (!this.OverviewMap.IsZoomedToMaxExtent)
            {
                var center = linearRing.Centroid.Coordinate;
                this.OverviewMap.ViewExtents.SetCenter(linearRing.Centroid.Coordinate);
            }

            this.OverviewMap.MapFrame.Invalidate();

        }

        private IMapLayerCollection DeserializeLayers(string fileName)
        {
            IMapLayerCollection layers = null;
            var layoutXmlDoc = new XmlDocument();
            var xmlDeserializer = new XmlDeserializer();

            try
            {
                var xml = File.ReadAllText(fileName);
                layoutXmlDoc.LoadXml(xml);
                var root = layoutXmlDoc.DocumentElement;
                var objs = xmlDeserializer.Deserialize<System.Object[]>(xml);
                if (objs.Length != 3)
                {
                    return layers;
                }

                var map = objs[1] as Map;
                layers = map.Layers;
            }
            catch (Exception ex)
            {
                logger.Error(ex);
            }

            return layers;
        }


        #endregion
    }
}
